#!/bin/bash
# Copyright (c) 2018, 2019 SAS Institute Inc, Cary, NC, USA, All Rights Reserved
#
# This script copies updated hotfix files from SASHome to the correct target
# directories in the designated LevNN subtree.
#

# chksrc - checks read access to specified file
# Returns 0 if NOT readable/found, 1 otherwise.
# (note this is backwards from typical true/false, but allows concatenation of conditionals like
#
#    chksrc foo && let missing++
#
# Version 1.1
# Renamed script
# Copy logCentralize.groovy also
#
# Version 1.2
# Changed SASHome sanity check, removed cp plugins to agent
#

SASEVMGRVER=2.3
MYNAME=`basename $0`

chksrc() {
  if [ -r "$1" ]; then
    return 1 
  else
    echo "Couldn't find/read expected source file:"
    echo "  $1"
    return 0 
  fi
}

# copysrc - conditionally copies files
#
# if $1 is "true", copy $2 to $3
#
# Returns 0 on success, 1 on failure
#
copysrc() {
  echo "cp $2 $3"

  if [ $1 == "true" ]; then
    cp $2 $3
    rc=$?
    if [ $rc != 0 ]; then
      echo "command failed, return code=$rc,"
      return 1
    fi
  fi
return 0
}

usage() {
  cat <<EOF

This script copies updated SAS hotfix files from the designated SASHome directory tree
to the target configuration directories in the specified LevNN subtree.

You must run this command as the SAS Install User.

 $MYNAME  [-doit] -home <yourSASHomeDir> -levn <yourConfigLevNDir>

 --doit              by default, the script shows what files it will copy without executing
                     the actual copy commands. The -doit option says to execute the copy commands.

 --home              (required) specify the full path of your SASHome directory, for example:
                        -home /opt/sas/SASHome

 --levn              (required) specify the full path of your Config/Lev<nn> directory, for example:
                        -levn /opt/sas/Config/Lev1

 -h | --h | --help   print this usage information

EOF

}


# Main execution path

# collect the arguments

excessargs=""
doit="false"
homedir=""
levn=""

while [ -n "$1" ]
do
  case "$1" in
    --home)
      shift
      homedir="$1"
      ;;

    --levn)
      shift
      levn="$1"
      ;;

    --doit)
      doit="true"
      ;;

    -h | --h | --help)
      shift
      usage
      exit 0
      ;;

     *)
# if we don't recognize argument as expected opt, explain and bail out
      echo "Unknown option specified: $1"
      usage
      exit 1
      ;;
  esac
  shift  # next value
done

date_time=`date +%F_%H-%M-%S`
echo $date_time
echo doit is $doit

# misc sanity checking
[ -z "$homedir" ] && echo && echo "Fatal, no --home <SASHome> option specified." && usage && exit 2 
[ -z "$levn" ] && echo && echo "Fatal, no --levn <LevNN> option specified." && usage && exit 2 

# does this look like a real SASHome?
if [ ! -e "$homedir/SASEnvironmentManagerAgent" ]; then
  echo "Fatal, this does not appear to be a valid SASHome directory:"
  echo "   $homedir"
  echo "A subdirectory we expect to exist there was not found."
  exit 3
fi

# does this look like a real LevNN?
if [ ! -e "$levn/sas.servers" ]; then
  echo "Fatal, this does not appear to be a valid LevNN directory:"
  echo "   $levn"
  echo "A script that we expect to exist there was not found."
  exit 4 
fi

#
# some of these copy operations only apply to the emi-framework host, so check for that
#

is_framework="false"
is_client="false"
is_hqserver="false"
is_hqagent="false"

[ -e "$levn/Web/SASEnvironmentManager/emi-framework" ] && is_framework="true" && echo "emi-framework detected"
[ -e "$levn/Web/SASEnvironmentManager/emi-client" ] && is_client="true" && echo "emi-client detected"
[ -e "$levn/Web/SASEnvironmentManager/server-5.8.0-EE" ] && is_hqserver="true" && echo "hq-server detected"
[ -e "$levn/Web/SASEnvironmentManager/agent-5.8.0-EE" ] && is_hqagent="true" && echo "hq-agent detected"

CMD="echo \# cp"
[ $doit == "true" ] && CMD="cp"

missing=0

if [ $is_framework == "true" ]; then
  echo
  echo "Checking emi-framework sources"
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/NetworkUtil.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/Validator.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/HQApiUtil.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/SASMacros/loadappcfg.sas" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/SASMacros/logarmanalyzer.sas" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/masterACMETL.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/masterAPMETL.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/logCentralize.groovy" && let missing++
fi

if [ $is_client ==  "true" ]; then
  echo "Checking emi-client sources"
  chksrc "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/classes/NetworkUtil.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/classes/HQApiUtil.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/configureAPM.groovy" && let missing++
  chksrc "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/logDiscovery.groovy" && let missing++
fi

if [ $is_hqserver == "true" ]; then
  echo "Checking JAR sources"
  chksrc "$homedir/SASEnvironmentManagerEnablementKitPlugin/${SASEVMGRVER}/emi-framework-plugin.jar" && let missing++
  chksrc "$homedir/SASEnvironmentManagerEnablementKitPlugin/${SASEVMGRVER}/sas-deploy-agent-plugin.jar" && let missing++
fi

if [ $missing -gt 0 ]; then
  echo
  echo "Fatal, $missing of the expected input files were not found. Giving up"
  exit 4
fi

####
# copy (or NO-OP echo) files
####

if [ $doit != "true" ]; then
  echo
  echo "Showing copy commands that will be executed."
fi

if [ $is_framework == "true" ]; then
  echo

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/NetworkUtil.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/classes/NetworkUtil.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/Validator.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/classes/Validator.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/classes/HQApiUtil.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/classes/HQApiUtil.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/SASMacros/loadappcfg.sas" \
    "$levn/Web/SASEnvironmentManager/emi-framework/SASMacros/loadappcfg.sas"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/SASMacros/logarmanalyzer.sas" \
    "$levn/Web/SASEnvironmentManager/emi-framework/SASMacros/logarmanalyzer.sas"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/masterACMETL.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/masterACMETL.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/masterAPMETL.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/masterAPMETL.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitServer/${SASEVMGRVER}/core/Scripts/logCentralize.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-framework/Scripts/logCentralize.groovy"
fi

if [ $is_client ==  "true" ]; then
  echo

  copysrc "$doit" "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/classes/NetworkUtil.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-client/Scripts/classes/NetworkUtil.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/classes/HQApiUtil.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-client/Scripts/classes/HQApiUtil.groovy"  

  copysrc "$doit" "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/configureAPM.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-client/Scripts/configureAPM.groovy"

  copysrc "$doit" "$homedir/SASEnvironmentManagerLogFileCollector/${SASEVMGRVER}/emi_client/Scripts/logDiscovery.groovy" \
    "$levn/Web/SASEnvironmentManager/emi-client/Scripts/logDiscovery.groovy"
fi

if [ $is_hqserver == "true" ]; then
  echo

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitPlugin/${SASEVMGRVER}/emi-framework-plugin.jar" \
    "$levn/Web/SASEnvironmentManager/server-5.8.0-EE/hq-engine/hq-server/webapps/ROOT/WEB-INF/hq-plugins/emi-framework-plugin.jar"

  copysrc "$doit" "$homedir/SASEnvironmentManagerEnablementKitPlugin/${SASEVMGRVER}/sas-deploy-agent-plugin.jar" \
    "$levn/Web/SASEnvironmentManager/server-5.8.0-EE/hq-engine/hq-server/webapps/ROOT/WEB-INF/hq-plugins/sas-deploy-agent-plugin.jar"
fi


if [ $doit != "true" ]; then
  echo
  echo "To execute, rerun this command adding the '--doit' option."
fi


